/*
    *
    * This file is a part of CoreStuff.
    * An activity viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QPainter>
#include <QDebug>
#include <QScreen>
#include <QTimer>
#include <QPluginLoader>
#include <QDateTime>
#include <QMouseEvent>
#include <QScroller>

#include <cprime/cplugininterface.h>
#include <cprime/cprime.h>
#include <cprime/corexdg.h>

#include "pagehome.h"
#include "ui_pagehome.h"


pagehome::pagehome(QWidget *parent) : QWidget(parent), ui(new Ui::pagehome)
{
    ui->setupUi(this);

    loadSettings();
    startSetup();
    QScreen *scrn = QGuiApplication::primaryScreen();
    setupWidgetView(scrn->availableGeometry());

}

pagehome::~pagehome()
{
    delete ui;
}

/**
 * @brief Loads application settings
 */
void pagehome::loadSettings()
{
    touch = sm->value("CoreApps", "TouchMode");
    desktop = sm->value("CoreStuff", "Desktop");
    tabletMode = sm->value("CoreApps", "TabletMode");
    selectedPlugins.clear();

    int count = sm->value("CoreAction", "SelectedPluginsCount");

    for (int i = 0; i < count; i++) {
        selectedPlugins << QString(sm->value("CoreAction", "SelectedPlugins/" + QString::number(i)));
    }
}

void pagehome::setupWidgetView(QRect geometry)
{
    int width = geometry.width();
    int height = geometry.height();
    int x = 0;

    if( width > height ) { // Landscape

        qDebug()<<"orientation: Landscape" ;
        if(touch){
            if(not tabletMode){  // mobile landscape
                qDebug()<<"Landscape: mobile";
                x = static_cast<int>(width * .77);
                ui->sp->changeSize(QSizePolicy::Fixed, QSizePolicy::Expanding);
                ui->time->hide();
                ui->day->show();
            }
            else{
                qDebug()<<"Landscape: tablet";
                x = static_cast<int>(width * .23);
                ui->day->hide();
                ui->time->show();
            }
        }
        else{
            qDebug()<<"Landscape: desktop";
            x = static_cast<int>(width * .23);
            ui->day->hide();
            ui->time->show();
        }
    }
    else {          // Portrait

        qDebug()<<"orientation: Portrait" ;
        if(touch){
            if(not tabletMode){
                qDebug()<<"Portait: mobile";
                x = static_cast<int>(width);
                ui->day->hide();
                ui->time->show();
            }
            else{
                qDebug()<<"Portait: tablet ";
                x = static_cast<int>(width * .77);
                ui->time->hide();
                ui->day->show();
            }
        }
        else{
            qDebug()<<"Portait: desktop";
            x = static_cast<int>(width * .77);
            ui->time->hide();
            ui->day->show();
        }
    }

    // set coreaction widith
    ui->widgetsL->setFixedWidth(x - 33);
    ui->widgets->setFixedWidth(x - 4);
    ui->frame->setFixedWidth( x );
}

void pagehome::startSetup()
{
    // Detect screen rotation and take necessary steps
    QScreen *scrn = qApp->primaryScreen();//qApp->screens()[screenIndex];

    connect(scrn, &QScreen::availableGeometryChanged, [this, scrn](const QRect & geometry) {
        if (desktop) {
            qDebug() << "Screen size Changed to: " << scrn->size();
            setupWidgetView(geometry);
        }
    });

//    // get the orentation
//    QScreen *scr = QGuiApplication::primaryScreen();
//    int width = scr->availableGeometry().width();
//    int height = scr->availableGeometry().height();
//    int x = 0;
//    bool orentation;


//    if(height>width){   // Portrait
//        orentation = 0;
//    } else{             // Landscape
//        orentation = 1;
//    }

//    if(orentation){ // Landscape

//        qDebug()<<"orientation: Landscape" ;
//        if(touch){
//            if(not tabletMode){  // mobile landscape
//                qDebug()<<"Landscape: mobile";
//                x = static_cast<int>(width * .80);
//                ui->sp->changeSize(QSizePolicy::Fixed, QSizePolicy::Expanding);
//                ui->time->hide();
//            }
//            else{
//                qDebug()<<"Landscape: tablet";
//                x = static_cast<int>(width * .23);
//                ui->day->hide();
//            }
//        }
//        else{
//            qDebug()<<"Landscape: desktop";
//            x = static_cast<int>(width * .23);
//            ui->day->hide();
//        }
//    }
//    else{          // Portrait

//        qDebug()<<"orientation: Portrait" ;
//        if(touch){
//            if(not tabletMode){
//                qDebug()<<"Portait: mobile";
//                x = static_cast<int>(width);
//                ui->day->hide();
//            }
//            else{
//                qDebug()<<"Portait: tablet ";
//                x = static_cast<int>(width * .80);
//                ui->time->hide();
//            }
//        }
//        else{
//            qDebug()<<"Portait: desktop";
//            x = static_cast<int>(width * .80);
//            ui->time->hide();
//        }
//    }


//    // set coreaction widith
//    ui->widgetsL->setFixedWidth(x - 33);
//    ui->widgets->setFixedWidth(x - 4);
//    ui->frame->setFixedWidth(x - 6);



    if (touch)
        QScroller::grabGesture(ui->widgets, QScroller::LeftMouseButtonGesture);


    updateTime();
    QTimer *timer = new QTimer(this);
    connect(timer, &QTimer::timeout, this, &pagehome::updateTime);
    timer->start(60000);

    if (!selectedPlugins.count()) {
        CPrime::InfoFunc::messageEngine(
            "CoreStuff", "corestuff", "Plugins not found",
            "Please install plugins and ensure they are enabled in CoreGarage.",
            this
        );
    }

    Q_FOREACH (QString pPath, selectedPlugins) {
        QPluginLoader loader(pPath);
        QObject *pObject = loader.instance();

        if (pObject) {
            plugininterface *plugin = qobject_cast<plugininterface *>(pObject);

            if (!plugin) {
                qWarning() << "Plugin Error:" << loader.errorString();
                continue;
            }

            QWidget *w = plugin->widget(this);
            ui->widgetLayout->addWidget(w);
        } else {
            qWarning() << "Plugin Error:" << loader.errorString();
        }
    }

    ui->widgetLayout->addStretch();
}

void pagehome::updateTime()
{

    QDateTime dt = QDateTime::currentDateTime();
    ui->day->setText( "<span style=font-size:24pt;>" +dt.toString("hh:mm AP") + "</span><br/><span style=font-size:18pt;>" + dt.toString("ddd MMMM d ") + "</span><br/>" + qgetenv("USER"));
    ui->time->setText( "<span style=font-size:24pt;>" +dt.toString("hh:mm AP") + "</span><br/><span style=font-size:18pt;>" + dt.toString("ddd MMMM d ") + "</span><br/>" );
}
