/*
    *
    * This file is a part of CoreStuff.
    * An activity viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QLabel>
#include <QScreen>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QApplication>

#include "global.h"

#include "powerdlg.h"

#include <cprime/settingsmanage.h>


PowerDialog::PowerDialog( QWidget *parent ) : QDialog( parent ) {

	setWindowFlags( Qt::Dialog | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint );
	setWindowModality( Qt::ApplicationModal );

    QSize toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");

	powerDBus = new Power( this );

	logOutBtn = new QCommandLinkButton( "Logout", "Logout as the current user" );
	logOutBtn->setIcon( QIcon::fromTheme( "system-log-out" ) );
    logOutBtn->setIconSize(toolBarIconSize);
	connect( logOutBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session
		parent->close();			// We do not want corestuff to appear while saving the session

		QProcess proc;
		proc.start( "desq-session-x11", QStringList() << "--logout" );
		proc.waitForFinished();

		/* We manually close this app */
		qApp->quit();
	} );

	suspendBtn = new QCommandLinkButton( "Suspend", "Suspend the system to RAM" );
	suspendBtn->setIcon( QIcon::fromTheme( "system-suspend" ) );
    suspendBtn->setIconSize(toolBarIconSize);
	connect( suspendBtn, &QCommandLinkButton::clicked, powerDBus, &Power::systemSuspend );
	if ( not powerDBus->systemCanSuspend() )
        suspendBtn->setDisabled( true );

	hibernateBtn = new QCommandLinkButton( "Hibernate", "Suspend the system to Disk" );
	hibernateBtn->setIcon( QIcon::fromTheme( "system-suspend-hibernate" ) );
    hibernateBtn->setIconSize(toolBarIconSize);
	connect( hibernateBtn, &QCommandLinkButton::clicked, powerDBus, &Power::systemHibernate );
	if ( not powerDBus->systemCanHibernate() )
        hibernateBtn->setDisabled( true );

    powerOffBtn = new QCommandLinkButton( "Power Off", "Power off the system" );
	powerOffBtn->setIcon( QIcon::fromTheme( "system-shutdown" ) );
    powerOffBtn->setIconSize(toolBarIconSize);
	connect( powerOffBtn, &QCommandLinkButton::clicked, powerDBus, &Power::systemHalt );
	if ( not powerDBus->systemCanHalt() )
        powerOffBtn->setDisabled( true );

    rebootBtn = new QCommandLinkButton( "Reboot", "Reboot the system" );
	rebootBtn->setIcon( QIcon::fromTheme( "system-reboot" ) );
    rebootBtn->setIconSize(toolBarIconSize);
	connect( rebootBtn, &QCommandLinkButton::clicked, powerDBus, &Power::systemReboot );
	if ( not powerDBus->systemCanReboot() )
        rebootBtn->setDisabled( true );

    cancelBtn = new QCommandLinkButton( "Cancel", "Do nothing" );
	cancelBtn->setIcon( QIcon::fromTheme( "dialog-close" ) );
    cancelBtn->setIconSize(toolBarIconSize);
	connect( cancelBtn, &QToolButton::clicked, this, &QDialog::reject );

    QVBoxLayout *lyt = new QVBoxLayout();
    lyt->setContentsMargins(5,5,5,5);
	lyt->addWidget( logOutBtn );
	lyt->addWidget( suspendBtn );
	lyt->addWidget( hibernateBtn );
	lyt->addWidget( powerOffBtn );
	lyt->addWidget( rebootBtn );
    lyt->addWidget( cancelBtn );

	setLayout( lyt );

    QSize scrnSize = QApplication::primaryScreen()->size();
    int h = ( scrnSize.width() - 250 ) / 2;
    int v = ( scrnSize.height() - 384 ) / 2;

    resize( scrnSize.width() * .15 , scrnSize.height() * .4);
    move( h, v );

    if (sm->value("CoreApps", "TouchMode")) {
        if (not sm->value("CoreApps", "TabletMode")){
            this->setWindowState(Qt::WindowMaximized);
        }
    }

};
